## Power of Two 解题报告

这是一道难度较高的分类讨论 + 贪心 + 构造题。

### 定义

- 题设：$x$ 表示 AND 的数量，$y$ 表示 OR 的数量，$z$ 表示 XOR 的数量。

- 题目表述为从 $c = 0$ 开始，不断将 $c$ AND / OR / XOR 二的幂，最大化 $c$ 的最终值，设为 $s$。

- 设 $A$ 表示所有出现的数的集合。
- 设 $M$ 表示所有出现次数 $> 1$ 的数的集合。
- 设 $cnt_i$ 表示 $i$ 的出现次数。

- 称 **点亮** 一个二进制位表示将其从 $0$ 变成 $1$。
- 称 **熄灭** 一个二进制位表示将其从 $1$ 变成 $0$。

### Case 1

AND 不会点亮 $c$ 的任何位，所以 $y + z$ 是 $\mathrm{popcount}(s)$ 的最大值。

当 $y + z \leq |A|$ 时，将所有 AND 放在前面，用 $y$ 个 OR 和 $z$ 个 XOR 点亮可能的最高的 $y + z$ 个二进制位，即 $A$ 的前 $y + z$ 大的数。

- 接下来有 $y + z > |A|$，它隐含条件 $|M| > 0$。

### Case 2

AND 的数量和 $|M|$ 对答案有很大影响，枚举它们，分类讨论。

当 $x = 0$ 时，OR 比 XOR 更厉害。只要让每个数最后一次被 OR 操作，那么对应位一定为 $1$。对于 $cnt_i$ 为奇数的 $i$，就算不分配 OR 也可以让对应位为 $1$，反之必须分配至少一个 OR。

综上，从大到小遍历 $A$，若当前位 $i$ 出现偶数次且还有 OR，则在它最后一次出现之前分配一个 OR，否则全部用 XOR 操作，如果 XOR 不够就用 OR。

- 接下来有 $x > 0$。

### Case 3

主要思路为 **在最后用 XOR 和 OR 点亮所有位**，称为 **填充**。为保证过程中 XOR 不会熄灭任何位，需要满足填充前 $c = 0$。填充过程前的所有操作称为 **准备**。

**如果填充可以成功进行，则每个出现过的位均被点亮，$s$ 取到最大值。**

为填充，将每个 $cnt_i(i\in A)$ 减去 $1$（接下来讨论修改后的 $cnt$），当前出现的数的集合为 $M$。因为 $y + z > |A|$，所以只要在填充前将所有 AND 用完，就一定有足够的 XOR 和 OR 进行填充，且填充前使用 $1$ 个 OR 或 XOR 是合法的。

当 $y > 0$ 时，考虑 $M$ 中任意一个数 $m$，只需在填充前最后一次操作 AND $2 ^ m$，并且填充时 OR $2 ^ m$ 即可。

- 接下来有 $y = 0$。

### Case 4

当 $x \geq 2$ 且 $|M| \geq 2$ 时，考虑 $M$ 中任意两个数 $m_1, m_2$，只需在填充前最后两次操作分别 AND $2 ^ {m_1}$ 和 AND $2 ^ {m_2}$ 即可。

- 接下来有 $x = 1$ 或 $|M| = 1$。

### Case 5

AND 和 XOR 的关键性质：AND $2 ^ m$ 不改变第 $m$ 位的状态，XOR $2 ^ m$ 一定改变第 $m$ 位的状态。

当 $|M| = 1$ 时，

#### Case 5.1

若 $z - |A|$ 是偶数，则准备部分有偶数个 XOR。根据性质，填充前 $c = 0$，合法。

#### Case 5.2

若 $z - |A|$ 是奇数，则准备部分有奇数个 XOR。因为 $A\backslash M$ 的每一位需要 XOR 点亮，所以此时不可能点亮所有位。

在一开始 AND $2 ^ {A_{\min}}$ 即可用 Case 5.1 的方法点亮其它所有位，最终 $s$ 只有第 $A_{\min}$ 位为 $0$。

### Case 6

通过上述所有分类讨论，现在只需处理 $x = 1$，$y = 0$，$z = n - 1$ 且 $|M| \geq 2$ 的情况。

#### Case 6.1

若存在 $m\in M$ 使得 $cnt_m$ 是奇数，只需在填充前最后一次操作 AND $2 ^ m$ 即可，因为操作前第 $m$ 位为 $0$。

#### Case 6.2

对于任意 $m\in M$ 均有 $cnt_m$ 是偶数，相当于原问题所有数出现奇数次。因为唯一的 AND $2 ^ v$ 不改变第 $v$ 位的值，所以此时不可能点亮所有位。

在一开始 AND $2 ^ {A_\min}$ 即可点亮其它所有位，最终 $s$ 只有第 $A_\min$ 位为 $0$。

综上，时间复杂度 $\mathcal{O}(n)$。





